#!/usr/bin/python
# -*- coding: utf-8 -*-

__version__ = '$Id: sss.py 21 2007-05-16 15:20:10Z rapha $'
__author__ = 'Raphaël Bollen'
__license__ = 'GPL V2.0'
__url__ = 'none yet!'
__doc__ = \
"""Read .sss.conf file and launch audio application according to file
specifications.

Copyright %s (2007)
Version: %s
License: %s
Homepage: %s

""" % (__author__,__version__,__license__,__url__)


import os
import subprocess
import ConfigParser
import optparse
import time
import threading
import getpass

import re

reg_passwd = re.compile("%PASSWORD%")

CWD = os.getcwd()
APPWD = os.path.abspath(os.path.join(CWD, os.path.dirname(__file__)))

def expect_run(py_script):
    execfile(py_script)

def subst_password(s):
    return reg_passwd.subn(opts.password, s, 0)[0]

def main():
    # check directory exist
    c = ConfigParser.ConfigParser()
    if opts.filename:
        c.readfp(open(opts.filename))
    else:
        c.readfp(open(os.path.expanduser("~/.sss.conf")))

    # kill running apps
    if c.has_option("DEFAULT", "killall"):
        for app in c.defaults()['killall'].split(";"):
            if c.has_option(app, "signal"):
                signal = c.get(app, "signal")
            else:
                signal = "SIGTERM"
            cmd = "killall -s %s %s" % (signal, app)
            if opts.verbose:
                print cmd
            try:
                pid = subprocess.Popen(args=cmd.split(),\
                                       stdin=subprocess.PIPE,\
                                       stdout=subprocess.PIPE,\
                                       stderr=subprocess.PIPE)
                data, err = pid.communicate()
                retcode = pid.wait()
                print data, err
                if retcode or err:
                    if c.has_option("DEFAULT", "stop_on_fail"):
                        if c.getboolean("DEFAULT", "stop_on_fail"):
                            raise RuntimeError(err)
                    else:
                        raise RuntimeError(err)
            except OSError, e:
                    print "Execution failed:", e
    
    # stop services
    if c.has_option("DEFAULT", "services"):
        for service in c.defaults()['services'].split(";"):
            cmd = "sudo -S /etc/init.d/%s stop" % service
            if opts.verbose:
                print cmd
            try:
                pid = subprocess.Popen(args=cmd.split(),\
                                       stdin=subprocess.PIPE,\
                                       stdout=subprocess.PIPE,\
                                       stderr=subprocess.PIPE)
                pid.stdin.write("%s\n" % opts.password)
                data, err = pid.communicate()
                retcode = pid.wait()
                print data, err
                if retcode or err:
                    if c.has_option("DEFAULT", "stop_on_fail"):
                        if c.getboolean("DEFAULT", "stop_on_fail"):
                            raise RuntimeError(err)
                    else:
                        raise RuntimeError(err)
            except OSError, e:
                    print "Execution failed:", e

    # expect scripts
    threads = []
    if c.has_option("DEFAULT", "expect"):
        for expect in c.defaults()['expect'].split(";"):
            if not c.has_section(expect):
                continue
            script = str(c.get(expect, "script"))
            print script
            if opts.verbose:
                print "Launching %s" % script
            if os.path.exists(os.path.abspath(script)):
                t = threading.Thread(target=expect_run,\
                                     kwargs={'py_script':script})
                threads.append(t)
                t.start()
            if c.has_option(expect, "timeout"):
                timeout = c.getfloat(expect, "timeout")
                if opts.verbose:
                    print "sleeping %s secs" % timeout
                time.sleep(timeout)

    # launch apps following launch_order
    pids = []
    reg = re.compile("(.*)`(\D*)\s(.*)`(.*)")
    for app in c.defaults()['launch_order'].split(';'):
        if not c.has_section(app):
            continue
        if c.has_option(app, "SUDO") and c.getboolean(app, "sudo"):
            cmd = "sudo -S %s %s" %(c.get(app, "cmd"), subst_password(c.get(app, "args")))
	    m = reg.search(cmd)
	    if m:
                res = subprocess.Popen(["%s" % m.group(2), "%s" % m.group(3)],\
                                       stdout=subprocess.PIPE).communicate()[0]
                cmd = m.group(1) + res + m.group(4)
            if opts.verbose:
                print "%s" % cmd
            p = subprocess.Popen(args=cmd.split(),\
                                 stdin=subprocess.PIPE,\
                                 stdout=subprocess.PIPE,\
                                 stderr=subprocess.PIPE)
            p.stdin.write("%s\n" % opts.password)
            data, err = p.communicate()
            time.sleep(1.0)
            #retcode = p.wait()
            if err:
                if c.has_option(app, "stop_on_fail"):
                    if c.getboolean(app, "stop_on_fail"):
                        raise RuntimeError(err)
                else:
                    raise RuntimeError(err)
            pids.append(p)
        else:
            cmd = "%s %s" %(c.get(app, "cmd"), subst_password(c.get(app, "args")))
            if opts.verbose:
                print cmd
            p = subprocess.Popen(args=cmd.split())
            pids.append(p)
        if c.has_option(app, "timeout"):
            timeout = c.getfloat(app, "timeout")
            if opts.verbose:
                print "sleeping %s secs" % timeout
            time.sleep(timeout)

    # wait for all pids to terminate
    if not c.getboolean("DEFAULT", "dont_wait"):
        stop = False
        while not stop:
            time.sleep(1)
            stop = True
            for pid in pids:
                if pid.poll() == None:
                    stop = False
    for thread in threads:
        thread.join()

    # restarting services
    if c.has_option("DEFAULT", "services"):
        for service in c.defaults()['services'].split(";"):
            cmd = "sudo -S /etc/init.d/%s start" % service
            if opts.verbose:
                print cmd
            try:
                pid = subprocess.Popen(args=cmd.split(),\
                                       stdin=subprocess.PIPE,\
                                       stdout=subprocess.PIPE,\
                                       stderr=subprocess.PIPE)
                pid.stdin.write("%s\n" % opts.password)
                data, err = pid.communicate()
                retcode = pid.wait()
                print data, err
            except OSError, e:
                    print "Execution failed:", e


    # process FINALLY section if any
    if c.has_option("DEFAULT", "finally"):
        for app in c.defaults()['finally'].split(';'):
            if not c.has_section(app):
                continue
            if c.has_option(app, "SUDO") and c.getboolean(app, "sudo"):
                cmd = "sudo -S %s %s" %(c.get(app, "cmd"),\
                                        subst_password(c.get(app, "args")))
                if opts.verbose:
                    print "%s" % cmd
                p = subprocess.Popen(args=cmd.split(),\
                                     stdin=subprocess.PIPE)
                p.stdin.write("%s\n" % opts.password)
            else:
                cmd = "%s %s" %(c.get(app, "cmd"), subst_password(c.get(app, "args")))
                if opts.verbose:
                    print cmd
                p = subprocess.Popen(args=cmd.split())
            if c.has_option(app, "timeout"):
                timeout = c.getfloat(app, "timeout")
                if opts.verbose:
                    print "sleeping %s secs" % timeout
                time.sleep(timeout)

    print "Done."
    

if __name__ == '__main__':
    # process options
    usage = "usage: %prog [options]"
    o = optparse.OptionParser(usage)
    o.add_option("-v", "--verbose", action="store_true", dest="verbose", \
                 help="Detailed script output.")
    o.add_option("-f", "--filename", action="store", type="string", \
                 dest="filename", \
                 help="Read setting from this file rather than default file.")
    o.add_option("-p", "--password", action="store", type="string", \
                 dest="password", \
                 help="User password for sudo.")

    global opts, args
    (opts, args) = o.parse_args()

    if  not opts.password:
        opts.password = getpass.getpass()

    main()
    
    
